"""Terminate an edge port in the network."""

from typing import Any

from orchestrator import workflow
from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import SubscriptionLifecycle
from orchestrator.workflow import StepList, begin, done, step
from orchestrator.workflows.steps import resync, set_status, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form
from pydantic_forms.types import FormGenerator, UUIDstr

from gso.products.product_types.edge_port import EdgePort
from gso.services.lso_client import execute_playbook, lso_interaction
from gso.services.netbox_client import NetboxClient
from gso.utils.types.tt_number import TTNumber


def initial_input_form_generator() -> FormGenerator:
    """Let the operator decide whether to delete configuration on the router, and clear up :term:`IPAM` resources."""

    class TerminateForm(FormPage):
        tt_number: TTNumber

    user_input = yield TerminateForm
    return user_input.model_dump()


@step("[DRY RUN] Remove Edge Port")
def remove_edge_port_dry(
    subscription: dict[str, Any], tt_number: str, process_id: UUIDstr, callback_route: str
) -> dict[str, Any]:
    """Remove an edge port from the network."""
    extra_vars = {
        "subscription": subscription,
        "dry_run": True,
        "verb": "terminate",
        "commit_comment": f"GSO_PROCESS_ID: {process_id} - TT_NUMBER: {tt_number} - Delete Edge Port",
    }

    execute_playbook(
        playbook_name="edge_port.yaml",
        callback_route=callback_route,
        inventory=subscription["edge_port"]["edge_port_node"]["router_fqdn"],
        extra_vars=extra_vars,
    )
    return {"subscription": subscription}


@step("[FOR REAL] Remove Edge Port")
def remove_edge_port_real(
    subscription: dict[str, Any], tt_number: str, process_id: UUIDstr, callback_route: str
) -> None:
    """Remove an edge port from the network."""
    extra_vars = {
        "subscription": subscription,
        "dry_run": False,
        "verb": "terminate",
        "commit_comment": f"GSO_PROCESS_ID: {process_id} - TT_NUMBER: {tt_number} - Delete Edge Port",
    }

    execute_playbook(
        playbook_name="edge_port.yaml",
        callback_route=callback_route,
        inventory=subscription["edge_port"]["edge_port_node"]["router_fqdn"],
        extra_vars=extra_vars,
    )


@step("Netbox Clean Up")
def netbox_clean_up(subscription: EdgePort) -> None:
    """Update Netbox to remove the edge port LAG interface and all the LAG members."""
    nbclient = NetboxClient()

    for member in subscription.edge_port.edge_port_ae_members:
        nbclient.free_interface(subscription.edge_port.edge_port_node.router_fqdn, member.interface_name)

    nbclient.delete_interface(subscription.edge_port.edge_port_node.router_fqdn, subscription.edge_port.edge_port_name)


@workflow(
    "Terminate Edge Port",
    initial_input_form=wrap_modify_initial_input_form(initial_input_form_generator),
    target=Target.TERMINATE,
)
def terminate_edge_port() -> StepList:
    """Terminate a new edge port in the network."""
    return (
        begin
        >> store_process_subscription(Target.TERMINATE)
        >> unsync
        >> lso_interaction(remove_edge_port_dry)
        >> lso_interaction(remove_edge_port_real)
        >> netbox_clean_up
        >> set_status(SubscriptionLifecycle.TERMINATED)
        >> resync
        >> done
    )
